from getpass import getpass
from ldap_settings import LDAP_ACCESS_SECRET
import ldap

class InvalidCredentials(Exception):
    """Raised when we couldn't connect to the server with the provided logins."""
    pass

class ConnectionFailure(Exception):
    """An unknown failure occured when connecting to the server."""
    pass

class TimeoutException(Exception):
    """The request timed-out."""
    pass

class OperationFailed(Exception):
    """The operation failed unexpectedly."""
    pass

def _get_connection(uri, uid=None, password=None):
    """Connect with an uid and a password."""
    c = ldap.initialize(uri)
    try:
        if uid is None:
            # Anonymous auth
            c.simple_bind_s('', '')
        else:
            # Regular auth
            c.simple_bind_s(uid, password)
    except ldap.INVALID_CREDENTIALS:
        raise InvalidCredentials
    except ldap.LDAPError:
        raise ConnectionFailure

    return c

def connect(directory, user=None):
    """Connect with one of the LDAP users stored in the settings."""
    if user:
        (uid, password) = LDAP_ACCESS_SECRET[directory]['accounts'][user]
        if password is None:
            password = getpass("Enter password for account '{}':".format(uid))
    else:
        (uid, password) = (None, None)

    return _get_connection(LDAP_ACCESS_SECRET[directory]['uri'], uid, password)
